-- This file:
--   http://angg.twu.net/AGDA/2022Cats3.agda.html
--   http://angg.twu.net/AGDA/2022Cats3.agda
--           (find-angg "AGDA/2022Cats3.agda")
-- Author: Eduardo Ochs <eduardoochs@gmail.com>
-- Version: 20220129
-- Public domain.
--
-- ;; (find-angg "AGDA/find-agdatype.el")
-- (load       "~/AGDA/find-agdatype.el")
--
-- (find-eev "eev-pdflike.el" "change-default-viewer")
-- (defalias 'find-pdf-page 'find-pdftoolsr-page)
-- (defalias 'find-pdf-page 'find-xpdf-page)
--
-- (defun a () (interactive) (find-angg "AGDA/2022Cats3.agda"))

module 2022Cats3 where

open import Level
open import Relation.Binary.PropositionalEquality as Eq
open Eq.≡-Reasoning

postulate ext : ∀ {ℓ ℓ'} {A : Set ℓ} {B : A → Set ℓ'} {f g : (a : A) → B a}
                  → (∀ (a : A) → f a ≡ g a) → f ≡ g

module Cats3 where

record Cat {ℓ₀ ℓ₁ : Level} : Set (suc (ℓ₀ ⊔ ℓ₁)) where
  field
    Objs  : Set ℓ₀
    Hom   : Objs → Objs → Set ℓ₁
    id    : (D : Objs) → (Hom D D)
    _◦_   : {C D E : Objs}
            → (g : Hom D E)
            → (f : Hom C D)
            →     (Hom C E)
    idL   : {D E : Objs}
            → (g : Hom D E)
            → (g ≡ g ◦ (id D))
    idR   : {D E : Objs}
            → (g : Hom D E)
            → (g ≡ (id E) ◦ g)
    assoc : {C D E F : Objs}
            → (f : Hom C D)
            → (g : Hom D E)
            → (m : Hom E F)
            → (m ◦ (g ◦ f) ≡ (m ◦ g) ◦ f)

catSet : Cat {suc zero} {zero}
Cat.Objs  catSet = Set
Cat.Hom   catSet = λ A B → (A → B)
Cat.id    catSet = λ A → (λ a → a)
Cat._◦_   catSet = λ g f → (λ a → g (f a))
Cat.idL   catSet = λ f → refl
Cat.idR   catSet = λ f → refl
Cat.assoc catSet = λ f g h → refl

catSetH : Cat {suc zero} {zero}
Cat.Objs  catSetH = Set
Cat.Hom   catSetH = λ A B → (A → B)
Cat.id    catSetH = λ A → (λ a → a)
Cat._◦_   catSetH = λ g f → (λ a → g (f a))
Cat.idL   catSetH = λ f → refl
Cat.idR   catSetH = λ f → refl
Cat.assoc catSetH = λ f g h → refl

record Functor {ℓ₀ ℓ₁ ℓ₂ ℓ₃ : Level}
               (catB : Cat {ℓ₀} {ℓ₁})
               (catA : Cat {ℓ₂} {ℓ₃})
             : Set (suc (ℓ₀ ⊔ ℓ₁ ⊔ ℓ₂ ⊔ ℓ₃)) where
  field
    ac0      : (Cat.Objs catB) → (Cat.Objs catA)
    ac1      : {C D : Cat.Objs catB}
               → (Cat.Hom catB C D)
               → (Cat.Hom catA (ac0 C) (ac0 D))
    respids  : {C : Cat.Objs catB}
               → (ac1 (Cat.id catB C) ≡ Cat.id catA (ac0 C))
    -- respcomp : {C D E : Cat.Objs catB}
    --            → {f : Cat.Hom catB C D}
    --            → {g : Cat.Hom catB D E}
    --            → (ac1 (Cat._◦_ catB g f) ≡ (Cat._◦_ catA (ac1 g) (ac1 f)))
  --
  -- R.ac0       C                  = RC
  -- R.ac1      {C} {D} f           = R f
  -- R.respids  {C}                 : R(id C) ≡ id(RC)
  -- R.respcomp {C} {D} {E} {f} {g} : R(g∘f) ≡ Rg∘Rf

record NT {ℓ₀ ℓ₁ ℓ₂ ℓ₃ : Level}
          {catC : Cat {ℓ₀} {ℓ₁}}
          {catD : Cat {ℓ₂} {ℓ₃}}
          (F G : Functor catC catD)
             : Set (suc (suc (ℓ₀ ⊔ ℓ₁ ⊔ ℓ₂ ⊔ ℓ₃))) where
  field
    ac       : (A : Cat.Objs catC)
               -> (Cat.Hom catD (Functor.ac0 F A) (Functor.ac0 G A))
    sqcond   : {A B : Cat.Objs catC}
               -> (f : Cat.Hom catC A B)
               -> ((Cat._◦_ catD (ac B) (Functor.ac1 F f)) ≡
                   (Cat._◦_ catD (Functor.ac1 G f) (ac A)))
               -> Set
  --
  -- T.ac      A        : Hom(FA,GA)
  -- T.sqcond {A} {B} f : TB∘Ff ≡ Gf∘TA

postulate catA : Cat {zero} {zero}
postulate catB : Cat {zero} {zero}
postulate R : Functor catB catA
postulate A : Cat.Objs catA
postulate C : Cat.Objs catB
postulate η : Cat.Hom catA A (Functor.ac0 R C)

postulate D E F : Cat.Objs catB
postulate f     : Cat.Hom  catB C D
postulate g     : Cat.Hom  catB D E

catB[C,-] : Functor catB catSetH
Functor.ac0      catB[C,-] D           = Cat.Hom catB C D
Functor.ac1      catB[C,-] {D} {E} g f = Cat._◦_ catB g f
Functor.respids  catB[C,-] {D} = trans (ext (λ f → sym (Cat.idR catB f))) refl
-- Functor.respcomp catB[C,-] {D} {E} {F} {g} {m} = {!!}

module On_DEFgm
    (D E F : Cat.Objs catB)
    (g : Cat.Hom catB D E)
    (m : Cat.Hom catB E F) where
  --
  idD          : Cat.Hom catB D D
  idD          = Cat.id  catB D
  B[C,D]       : Set
  B[C,D]       = Functor.ac0 catB[C,-] D
  B[C,idD]     : Cat.Hom catSet B[C,D] B[C,D]
  B[C,idD]     = Functor.ac1 catB[C,-] idD
  id[B[C,D]]   : Cat.Hom catSet B[C,D] B[C,D]
  id[B[C,D]]   = Cat.id  catSet B[C,D]
  λf⇒[idD◦f]   : (f : B[C,D]) -> B[C,D]
  λf⇒[idD◦f]   f = Cat._◦_ catB idD f
  λf⇒f         : (f : B[C,D]) -> B[C,D]
  λf⇒f         f = f
  λf⇒<idD◦f≡f> : (f : B[C,D]) → (Cat._◦_ catB idD f) ≡ f
  λf⇒<idD◦f≡f> f = sym (Cat.idR catB f)
  --
  <B[C,idD]≡[λf⇒[idD◦f]]> : B[C,idD] ≡ λf⇒[idD◦f]
  <B[C,idD]≡[λf⇒[idD◦f]]> = refl
  <[λf⇒[idD◦f]]≡[λf⇒f]>   : λf⇒[idD◦f] ≡ λf⇒f
  <[λf⇒[idD◦f]]≡[λf⇒f]>   = ext λf⇒<idD◦f≡f>
  <[λf⇒f]≡id[B[C,D]]>     : λf⇒f ≡ id[B[C,D]]
  <[λf⇒f]≡id[B[C,D]]>     = refl
  --
  <B[C,idD]≡id[B[C,D]]>  : B[C,idD] ≡ id[B[C,D]]
  <B[C,idD]≡id[B[C,D]]>  = begin
      B[C,idD]    ≡⟨ <B[C,idD]≡[λf⇒[idD◦f]]> ⟩
      λf⇒[idD◦f]  ≡⟨ <[λf⇒[idD◦f]]≡[λf⇒f]>   ⟩
      λf⇒f        ≡⟨ <[λf⇒f]≡id[B[C,D]]>     ⟩
      id[B[C,D]]
    ∎
  --
  B[C,E]       : Set
  B[C,E]       = Functor.ac0 catB[C,-] E
  B[C,F]       : Set
  B[C,F]       = Functor.ac0 catB[C,-] F
  B[D,E]       : Set
  B[D,E]       = Cat.Hom catB D E
  B[D,F]       : Set
  B[D,F]       = Cat.Hom catB D F
  B[E,F]       : Set
  B[E,F]       = Cat.Hom catB E F
  B[C,g]       : Cat.Hom catSet B[C,D] B[C,E]
  B[C,g]       f = Functor.ac1 catB[C,-] g f
  B[C,m]       : Cat.Hom catSet B[C,E] B[C,F]
  B[C,m]       k = Functor.ac1 catB[C,-] m k
  λk⇒m◦k            : B[C,E] → B[C,F]
  λk⇒m◦k            k = Cat._◦_ catB m k
  λf⇒g◦f            : B[C,D] → B[C,E]
  λf⇒g◦f            f = Cat._◦_ catB g f
  [λk⇒m◦k]◦[λf⇒g◦f] : B[C,D] → B[C,F]
  [λk⇒m◦k]◦[λf⇒g◦f] f = λk⇒m◦k (λf⇒g◦f f)
  λf⇒[m◦[g◦f]]      : B[C,D] → B[C,F]
  λf⇒[m◦[g◦f]]      f = Cat._◦_ catB m (Cat._◦_ catB g f)
  λf⇒[[m◦g]◦f]      : B[C,D] → B[C,F]
  λf⇒[[m◦g]◦f]      f = Cat._◦_ catB (Cat._◦_ catB m g) f
  g◦f               : (g : B[D,E]) → (f : B[C,D]) → B[C,E]
  g◦f               g f = Cat._◦_ catB g f
  m◦g               : (m : B[E,F]) → (g : B[D,E]) → B[D,F]
  m◦g               m g = Cat._◦_ catB m g
  m◦k               : (m : B[E,F]) → (k : B[C,E]) → B[C,F]
  m◦k               m k = Cat._◦_ catB m k
  m◦[g◦f]           : (m : B[E,F]) → (g : B[D,E]) → (f : B[C,D]) → B[C,F]
  m◦[g◦f]           m g f = Cat._◦_ catB m (g◦f g f)
  [m◦g]◦f           : (m : B[E,F]) → (g : B[D,E]) → (f : B[C,D]) → B[C,F]
  [m◦g]◦f           m g f = Cat._◦_ catB (m◦g m g) f
  m◦[g◦f]≡[m◦g]◦f   : (m : B[E,F]) → (g : B[D,E]) → (f : B[C,D])
                      → (m◦[g◦f] m g f ≡ [m◦g]◦f m g f)
  m◦[g◦f]≡[m◦g]◦f   m g f = Cat.assoc catB f g m

  -- m◦[g◦f]           : (g : B[D,E]) → (m : B[E,F]) → 
  -- m◦[g◦f]≡[m◦g]◦f   : (g : B[D,E]) → (m : B[E,F]) → 

  --
  -- [λk⇒m◦k]◦[λf⇒g◦f]
  -- = λf⇒[[[λk⇒m◦k]◦[λf⇒g◦f]][f]]
  -- = λf⇒[[λk⇒m◦k][[λf⇒g◦f][f]]]
  -- = λf⇒[[λk⇒m◦k][g◦f]]
  -- = λf⇒[m◦[g◦f]]
  -- = λf⇒[[m◦g]◦f]
  -- = Bλf⇒[[m◦g]◦f]


-- (find-es "agda" "module-system")

-- module M = On_DEF D E F -- where
-- open M

-- foo : {!Set!}
-- foo = <B[C,idD]≡id[B[C,D]]>
-- trans (ext (λ f → sym (Cat.idR catB f))) refl


  --
  -- [λf⇒[id:D◦f]]≡[λf⇒f] : λf⇒[id:D◦f] ≡ λf⇒f
  -- [λf⇒[id:D◦f]]≡[λf⇒f] = {!!}


-- Local Variables:
-- coding:  utf-8-unix
-- End:
