-- This file:
--   http://angg.twu.net/AGDA/2022Cats4.agda.html
--   http://angg.twu.net/AGDA/2022Cats4.agda
--           (find-angg "AGDA/2022Cats4.agda")
-- Author: Eduardo Ochs <eduardoochs@gmail.com>
-- Version: 20220125
-- Public domain.
--
-- ;; (find-angg "AGDA/find-agdatype.el")
-- (load       "~/AGDA/find-agdatype.el")
--
-- (find-eev "eev-pdflike.el" "change-default-viewer")
-- (defalias 'find-pdf-page 'find-pdftoolsr-page)
-- (defalias 'find-pdf-page 'find-xpdf-page)
--
-- (defun a () (interactive) (find-angg "AGDA/2022Cats4.agda"))
-- (defun e () (interactive) (find-angg "LATEX/2022Cats4.lagda.tex"))

module 2022Cats4 where

open import Level
open import Relation.Binary.PropositionalEquality as Eq
open Eq.≡-Reasoning

postulate ext : ∀ {ℓ ℓ'} {A : Set ℓ} {B : A → Set ℓ'} {f g : (a : A) → B a}
                  → (∀ (a : A) → f a ≡ g a) → f ≡ g

module Cats3 where

record Cat {ℓ₀ ℓ₁ : Level} : Set (suc (ℓ₀ ⊔ ℓ₁)) where
  field
    Objs  : Set ℓ₀
    Hom   : Objs → Objs → Set ℓ₁
    id    : (D : Objs) → (Hom D D)
    _◦_   : {C D E : Objs}
            → (g : Hom D E)
            → (f : Hom C D)
            →     (Hom C E)
    idL   : {D E : Objs}
            → (g : Hom D E)
            → (g ≡ g ◦ (id D))
    idR   : {D E : Objs}
            → (g : Hom D E)
            → (g ≡ (id E) ◦ g)
    assoc : {C D E F : Objs}
            → (f : Hom C D)
            → (g : Hom D E)
            → (m : Hom E F)
            → (m ◦ (g ◦ f) ≡ (m ◦ g) ◦ f)

catSet : Cat {suc zero} {zero}
Cat.Objs  catSet = Set
Cat.Hom   catSet = λ A B → (A → B)
Cat.id    catSet = λ A → (λ a → a)
Cat._◦_   catSet = λ g f → (λ a → g (f a))
Cat.idL   catSet = λ f → refl
Cat.idR   catSet = λ f → refl
Cat.assoc catSet = λ f g h → refl

catSetH : Cat {suc zero} {zero}
Cat.Objs  catSetH = Set
Cat.Hom   catSetH = λ A B → (A → B)
Cat.id    catSetH = λ A → (λ a → a)
Cat._◦_   catSetH = λ g f → (λ a → g (f a))
Cat.idL   catSetH = λ f → refl
Cat.idR   catSetH = λ f → refl
Cat.assoc catSetH = λ f g h → refl

record Functor {ℓ₀ ℓ₁ ℓ₂ ℓ₃ : Level}
               (catB : Cat {ℓ₀} {ℓ₁})
               (catA : Cat {ℓ₂} {ℓ₃})
             : Set (suc (ℓ₀ ⊔ ℓ₁ ⊔ ℓ₂ ⊔ ℓ₃)) where
  field
    ac0      : (Cat.Objs catB) → (Cat.Objs catA)
    ac1      : {C D : Cat.Objs catB}
               → (Cat.Hom catB C D)
               → (Cat.Hom catA (ac0 C) (ac0 D))
    respids  : {C : Cat.Objs catB}
               → (ac1 (Cat.id catB C) ≡ Cat.id catA (ac0 C))
    respcomp : {C D E : Cat.Objs catB}
               → {f : Cat.Hom catB C D}
               → {g : Cat.Hom catB D E}
               → (ac1 (Cat._◦_ catB g f) ≡ (Cat._◦_ catA (ac1 g) (ac1 f)))
  --
  -- How to use it:
  -- R.ac0       C                  = RC
  -- R.ac1      {C} {D} f           = R f
  -- R.respids  {C}                 : R(id C) ≡ id(RC)
  -- R.respcomp {C} {D} {E} {f} {g} : R(g∘f) ≡ Rg∘Rf

record NT {ℓ₀ ℓ₁ ℓ₂ ℓ₃ : Level}
          {catC : Cat {ℓ₀} {ℓ₁}}
          {catD : Cat {ℓ₂} {ℓ₃}}
          (F G : Functor catC catD)
             : Set (suc (suc (ℓ₀ ⊔ ℓ₁ ⊔ ℓ₂ ⊔ ℓ₃))) where
  field
    ac       : (A : Cat.Objs catC)
               -> (Cat.Hom catD (Functor.ac0 F A) (Functor.ac0 G A))
    sqcond   : {A B : Cat.Objs catC}
               -> (f : Cat.Hom catC A B)
               -> ((Cat._◦_ catD (ac B) (Functor.ac1 F f)) ≡
                   (Cat._◦_ catD (Functor.ac1 G f) (ac A)))
               -> Set
  --
  -- How to use it:
  -- T.ac      A        : Hom(FA,GA)
  -- T.sqcond {A} {B} f : TB∘Ff ≡ Gf∘TA

postulate catA : Cat {zero} {zero}
postulate catB : Cat {zero} {zero}
postulate R : Functor catB catA
postulate A : Cat.Objs catA
postulate C : Cat.Objs catB
postulate η : Cat.Hom catA A (Functor.ac0 R C)

postulate D E F : Cat.Objs catB
postulate f     : Cat.Hom  catB C D
postulate g     : Cat.Hom  catB D E

catB[C,-] : Functor catB catSetH
Functor.ac0      catB[C,-] D           = Cat.Hom catB C D
Functor.ac1      catB[C,-] {D} {E} g f = Cat._◦_ catB g f
Functor.respids  catB[C,-] {D} = {!!}
Functor.respcomp catB[C,-] {D} {E} {F} {g} {m} = {!!}

module On_D (D : Cat.Objs catB) where
 module On_f (f : Cat.Hom catB C D) where
  id:D         : Cat.Hom catB D D
  id:D         = Cat.id  catB D
  catB[C,D]    : Set
  catB[C,D]    = Functor.ac0 catB[C,-] D
  catB[C,E]    : Set
  catB[C,E]    = Functor.ac0 catB[C,-] E
  catB[C,F]    : Set
  catB[C,F]    = Functor.ac0 catB[C,-] F
  catB[C,id:D] : Cat.Hom catSet catB[C,D] catB[C,D]
  catB[C,id:D] = Functor.ac1 catB[C,-] id:D
  id:catB[C,D] : Cat.Hom catSet catB[C,D] catB[C,D]
  id:catB[C,D] = Cat.id  catSet catB[C,D]
  id:D◦f       : Cat.Hom catB C D
  id:D◦f       = Cat._◦_ catB id:D f
  <f≡[id:D◦f]> : f ≡ id:D◦f
  <f≡[id:D◦f]> = Cat.idR catB {C} {D} f
  <[id:D◦f]≡f> : id:D◦f ≡ f
  <[id:D◦f]≡f> = sym <f≡[id:D◦f]>
  λf⇒f         : {D : Cat.Objs catB} → (f : Cat.Hom catB C D) → Cat.Hom catB C D
  λf⇒f        {D} f  = f
  λf⇒[id:D◦f]  : {D : Cat.Objs catB} → (f : Cat.Hom catB C D) → Cat.Hom catB C D
  λf⇒[id:D◦f] {D} f = Cat._◦_ catB (Cat.id catB D) f
  λf⇒<idD◦f≡f> : {D : Cat.Objs catB} → (f : Cat.Hom catB C D)
                 → (Cat._◦_ catB (Cat.id catB D) f) ≡ f
  λf⇒<idD◦f≡f> {D} f = sym (Cat.idR catB {C} {D} f)
  <catB[C,id:D]≡[λf⇒[id:D◦f]]> : catB[C,id:D] ≡ λf⇒[id:D◦f] {D}
  <catB[C,id:D]≡[λf⇒[id:D◦f]]> = refl
  <[λf⇒[id:D◦f]]≡[λf⇒f]>       : λf⇒[id:D◦f] {D} ≡ λf⇒f {D}
  <[λf⇒[id:D◦f]]≡[λf⇒f]>       = ext λf⇒<idD◦f≡f>
  <[λf⇒f]≡id:catB[C,D]>        : λf⇒f {D} ≡ id:catB[C,D]
  <[λf⇒f]≡id:catB[C,D]>        = refl
  <catB[C,id:D]≡id:catB[C,D]>  : catB[C,id:D] ≡ id:catB[C,D]
  <catB[C,id:D]≡id:catB[C,D]>  = begin
      catB[C,id:D]    ≡⟨ <catB[C,id:D]≡[λf⇒[id:D◦f]]> ⟩
      λf⇒[id:D◦f] {D} ≡⟨ <[λf⇒[id:D◦f]]≡[λf⇒f]>       ⟩
      λf⇒f        {D} ≡⟨ <[λf⇒f]≡id:catB[C,D]>        ⟩
      id:catB[C,D]
    ∎
  <catB[C,id:D]≡id:catB[C,D]>2 : catB[C,id:D] ≡ id:catB[C,D]
  <catB[C,id:D]≡id:catB[C,D]>2 = trans (ext (λ (f : Cat.Hom catB C D)
                                               → sym (Cat.idR catB f)))
                                       refl

  --
  -- [λf⇒[id:D◦f]]≡[λf⇒f] : λf⇒[id:D◦f] ≡ λf⇒f
  -- [λf⇒[id:D◦f]]≡[λf⇒f] = {!!}

module M1 = On_D D
module M2 = M1.On_f f

foo : {!!}
foo = M2.<catB[C,id:D]≡id:catB[C,D]>


foo0 : {DD : Cat.Objs catB} → {!!}
foo0 {DD} = trans (ext (λ (ff : Cat.Hom catB C D) → sym (Cat.idR catB ff))) refl



-- Local Variables:
-- coding:  utf-8-unix
-- End:
