# This file:
#   http://anggtwu.net/MANIM/jovane1.py.html
#   http://anggtwu.net/MANIM/jovane1.py
#          (find-angg "MANIM/jovane1.py")
# Original author: Jovane Mafort
# Slightly modified by: Eduardo Ochs
# See: http://angg.twu.net/emacsconf2021.html
# Version: 2021oct25

from math import *
from manim import *
from numpy import *
from decimal import *

class Exemple1(Scene):
    def construct(self):
        axes = (
            Axes(
                x_range=[0,7,1],
                x_length=9,
                y_range=[0,2,1],
                y_length=3,
                axis_config={"include_numbers":True, "include_tip": False}
            )
            .to_edge(LEFT)
            .set_color(GREY)
        )
        axes_labels = axes.get_axis_labels(x_label='x', y_label='y')

        func = axes.get_graph(
            lambda x : sin(x), x_range=[0,2*pi], color=RED
        )

        x = ValueTracker(2)
        dx = ValueTracker(2.5)

        secant = always_redraw(
            lambda: axes.get_secant_slope_group(
                x=x.get_value(),
                graph=func,
                dx=dx.get_value(),
                dx_line_color=YELLOW,
                dy_line_color=BLUE,
                dx_label='dx',
                dy_label='dy',
                secant_line_color=GREEN,
                secant_line_length=6
            )
        )

        dot1 = always_redraw(
            lambda: Dot()
            .scale(0.7)
            .move_to(axes.c2p(x.get_value(), func.underlying_function(x.get_value())))
        )

        dot2 = always_redraw(
            lambda: Dot()
            .scale(0.7)
            .move_to(axes.c2p(
                x.get_value() + dx.get_value(), 
                func.underlying_function(x.get_value() + dx.get_value())))
        )

        info = always_redraw(
            lambda: DecimalNumber().set_value(cos(x.get_value()))
            .move_to(UR)
        )

        function_tex = Tex('f(x) = sin(x)').shift(5*LEFT + 3*UP)
        d2 = Tex("f''(x) = -sin(x)").shift(2*RIGHT+3*UP)

        self.add(axes, axes_labels, func, function_tex, d2)
        self.play(Create(VGroup(dot1, dot2, secant, info)))
        self.play(dx.animate.set_value(0.001), run_time=8)
        self.wait(2)
        self.play(x.animate.set_value(0.1), run_time=5)
        self.wait()
        self.play(x.animate.set_value(2*pi-0.3), run_time=5)
        self.wait()
        self.play(x.animate.set_value(pi), run_time=6)
        self.wait()

class Exemple2(Scene):
    def construct(self):
        axes = (
            Axes(
                x_range=[-1,3,1],
                x_length=8,
                y_range=[-1,3,1],
                y_length=4,
                axis_config={"include_numbers":True, "include_tip": False}
            )
            .to_edge(LEFT)
            .set_color(GREY)
        )
        axes_labels = axes.get_axis_labels(x_label='x', y_label='y')

        func = axes.get_graph(
            lambda x : cbrt(x-1), x_range=[-1,3], color=RED
        )

        x = ValueTracker(0.5)
        dx = ValueTracker(1)

        secant = always_redraw(
            lambda: axes.get_secant_slope_group(
                x=x.get_value(),
                graph=func,
                dx=dx.get_value(),
                dx_line_color=YELLOW,
                dy_line_color=BLUE,
                dx_label='dx',
                dy_label='dy',
                secant_line_color=GREEN,
                secant_line_length=6
            )
        )

        dot1 = always_redraw(
            lambda: Dot()
            .scale(0.7)
            .move_to(axes.c2p(x.get_value(), func.underlying_function(x.get_value())))
        )

        dot2 = always_redraw(
            lambda: Dot()
            .scale(0.7)
            .move_to(axes.c2p(
                x.get_value() + dx.get_value(), 
                func.underlying_function(x.get_value() + dx.get_value())))
        )

        info = always_redraw(
            lambda: DecimalNumber().set_value(1/(3*sqrt((x.get_value()-1)**2)))
            .move_to(UR)
        )

        function_tex = Tex('f(x) = $\sqrt[3]{x-1}$').shift(5*LEFT + 3*UP)
        d2 = Tex("f''(x) = -$\\frac{2}{9(x-1)^{\\frac{5}{3}}}$").shift(2*RIGHT+3*UP)


        self.add(axes, axes_labels, func, function_tex, d2)
        self.play(Create(VGroup(dot1, dot2, secant, info)))
        self.play(dx.animate.set_value(0.00001), run_time=8)
        self.wait(2)
        self.play(x.animate.set_value(-0.8), run_time=5)
        self.wait()
        self.play(x.animate.set_value(2.8), run_time=5)
        self.wait()
        self.play(x.animate.set_value(1), run_time=6)
        self.wait()

class Exemple3(Scene):
    def construct(self):
        axes = (
            Axes(
                x_range=[-1,3,1],
                x_length=8,
                y_range=[-1,3,1],
                y_length=4,
                axis_config={"include_numbers":True, "include_tip": False}
            )
            .to_edge(LEFT)
            .set_color(GREY)
        )
        axes_labels = axes.get_axis_labels(x_label='x', y_label='y')

        func = axes.get_graph(
            lambda x : (x-1)**2-1, x_range=[-1,3], color=RED
        )

        x = ValueTracker(0.5)
        dx = ValueTracker(1)

        secant = always_redraw(
            lambda: axes.get_secant_slope_group(
                x=x.get_value(),
                graph=func,
                dx=dx.get_value(),
                dx_line_color=YELLOW,
                dy_line_color=BLUE,
                dx_label='dx',
                dy_label='dy',
                secant_line_color=GREEN,
                secant_line_length=6
            )
        )

        dot1 = always_redraw(
            lambda: Dot()
            .scale(0.7)
            .move_to(axes.c2p(x.get_value(), func.underlying_function(x.get_value())))
        )

        dot2 = always_redraw(
            lambda: Dot()
            .scale(0.7)
            .move_to(axes.c2p(
                x.get_value() + dx.get_value(), 
                func.underlying_function(x.get_value() + dx.get_value())))
        )

        info = always_redraw(
            lambda: DecimalNumber().set_value(2*(x.get_value())-2)
            .move_to(UR)
        )

        function_tex = Tex('f(x) = $(x-1)^2-1$').shift(5*LEFT + 3*UP)
        d2 = Tex("f''(x) = 2").shift(2*RIGHT+3*UP)

        self.add(axes, axes_labels, func, function_tex, d2)
        self.play(Create(VGroup(dot1, dot2, secant, info)))
        self.play(dx.animate.set_value(0.001), run_time=8)
        self.wait(2)
        self.play(x.animate.set_value(-0.8), run_time=5)
        self.wait()
        self.play(x.animate.set_value(2.8), run_time=5)
        self.wait()
        self.play(x.animate.set_value(1), run_time=6)
        self.wait()

class Exemple4(Scene):
    def construct(self):
        axes = (
            Axes(
                x_range=[-2,2,1],
                x_length=8,
                y_range=[-2,2,1],
                y_length=4,
                axis_config={"include_numbers":True, "include_tip": False}
            )
            .to_edge(LEFT)
            .set_color(GREY)
        )
        axes_labels = axes.get_axis_labels(x_label='x', y_label='y')

        func = axes.get_graph(
            lambda x : x**3, x_range=[-2,2], color=RED
        )

        x = ValueTracker(-1)
        dx = ValueTracker(-0.5)

        secant = always_redraw(
            lambda: axes.get_secant_slope_group(
                x=x.get_value(),
                graph=func,
                dx=dx.get_value(),
                dx_line_color=YELLOW,
                dy_line_color=BLUE,
                dx_label='dx',
                dy_label='dy',
                secant_line_color=GREEN,
                secant_line_length=6
            )
        )

        dot1 = always_redraw(
            lambda: Dot()
            .scale(0.7)
            .move_to(axes.c2p(x.get_value(), func.underlying_function(x.get_value())))
        )

        dot2 = always_redraw(
            lambda: Dot()
            .scale(0.7)
            .move_to(axes.c2p(
                x.get_value() + dx.get_value(), 
                func.underlying_function(x.get_value() + dx.get_value())))
        )

        info = always_redraw(
            lambda: DecimalNumber().set_value(3*(x.get_value())**2)
            .move_to(UR)
        )

        function_tex = Tex('f(x) = $x^3$').shift(5*LEFT + 3*UP)
        d2 = Tex("f''(x) = 6x").shift(2*RIGHT+3*UP)

        self.add(axes, axes_labels, func, function_tex, d2)
        self.play(Create(VGroup(dot1, dot2, secant, info)))
        self.play(dx.animate.set_value(0.001), run_time=8)
        self.wait(2)
        self.play(x.animate.set_value(-1.8), run_time=5)
        self.wait()
        self.play(x.animate.set_value(1.8), run_time=5)
        self.wait()
        self.play(x.animate.set_value(0), run_time=6)
        self.wait()



"""
 (eepitch-python)
 (eepitch-kill)
 (eepitch-python)
exec(open("jovane1.py").read(), globals())

 (eepitch-shell)
 (eepitch-kill)
 (eepitch-shell)
manim -ql -p jovane1.py Exemple1

# (find-extra-file-links "~/MANIM/media/videos/jovane1/480p15/Exemple1.mp4" "exemple1")
# (code-video "exemple1video" "~/MANIM/media/videos/jovane1/480p15/Exemple1.mp4")
# (find-exemple1video)
# (find-exemple1video "0:00")

"""
