-- sequence:            (find-fline "78032.f")
-- fields (9 per line): (find-fline "78032.t")
-- gi file:             (find-fline "78032.gi")
-- (find-fline "/var/www/tmp/MARCOS/")
--  http://200.222.206.134/tmp/MARCOS/foo.lua
-- (find-fline "")

-- Note that due to my seeting for the env. var LUA_INIT,
--   # (find-angg ".zshrc" "lua")
--   export LUA_INIT=@/home/edrx/LUA/lua50init.lua
-- the Lua interpreter loads a file with load of defs before loading this:
--   # (find-angg "LUA/lua50init.lua")


#
cd ~/MARCOS/
cat > t.lua <<'%%%'
-- read_tfile(fname): read the file fname line by line, discarding
--   the blank lines and the "#" lines, and return a list of structs
--   like {field7_as_num, field8_as_num, field1, field2}.
-- Example: if the file fname has:
--     # Fields: 1=qid, 2=sid, 3=numid, 4=alignlen, 5=misms, 6=gopenings,
--     #         7=qstart, 8=qend, 9=sstart, 10=send, 11=e-value, 12=bitscore
--     gi|78032 gi|79254 x x x x 5748 6936 x x x x
--     gi|78032 gi|80253 x x x x 5839 6936 x x x x
-- then read_t_file(fname) returns:
--     {{5748, 6936, "gi|78032", "gi|79254"},
--      {5839, 6936, "gi|78032", "gi|80253"}}
read_tfile = function(fname)
    local A = {}
    for line in io.lines(fname) do
      if line ~= "" then
        if not string.find(line, "^#") then
          local fields = split(line)
          local qid,    sid  =          fields[1],           fields[2]
          local qstart, qend = tonumber(fields[7]), tonumber(fields[8])
          tinsert(A, {qstart, qend, qid, sid})
        end
      end
    end
    return A
  end

-- Comparison functions for table.sort.
-- (find-Lua50ref "table.sort")
-- comp1 uses only the first field,
-- comp2 uses the second field when the two first fields are equal.
comp1  = function(p1, p2) return p1[1] < p2[1] end
comp12 = function(p1, p2)
    if p1[1] ~= p2[1]
    then return p1[1] < p2[1]
    else return p1[2] < p2[2]
    end
  end

-- tfile_to_qstartqends(tfile): a filter for tfile structures, that
--   does two things: (1) uses the argument gi and the 3rd and 4th
--   fields of each record (the "gi|..." strings; see the description
--   of read_tfile above) to drop out the 4-uples in which these
--   fields are both equal to gi; and (2) keep only the 1st and 2nd
--   fields, reducing each 4-uple to a pair (of numbers; see above).
--   This function returns a new structure and doesn't change tfile.
tfile_to_qstartqends = function(tfile, gi)
    local B = {}
    for i=1,table.getn(tfile) do
      local fields = tfile[i]
      if not (fields[3] == gi and fields[4] == gi) then
        tinsert(B, {fields[1], fields[2]})
      end
    end
    return B
  end

-- qstartqends_to_ovps: converts an ordered list of intervals (an
-- ordered "qstartqends") into an "ovps", which is an ordered list of
-- the ranges of ovelapping intervals in the qstartqends. Example:
--   qstartqends = {{1, 4}, {3, 7}, {10, 12}, {13, 16}, {20, 30}}
--      -> ovps = {{1,         7}, {10,           16}, {20, 30}}
qstartqends_to_ovps = function (qstartqends)
    local C = {}
    local ovpstart, ovpend
    for i=1,getn(qstartqends) do
      local pair = qstartqends[i]
      if not ovpstart then
        ovpstart, ovpend = pair[1], pair[2]
      end
      if pair[1] <= ovpend+1 then            -- if pair intersects ovp
        ovpend = max(ovpend, pair[2])        -- then extend ovp
      else
        tinsert(C, {ovpstart, ovpend})       -- else store this ovp
        ovpstart, ovpend = pair[1], pair[2]  -- and copy pair->ovp
      end
    end
    if ovpstart then                         -- we may need to
      tinsert(C, {ovpstart, ovpend})         -- flush the last interval
    end
    return C
  end

-- (find-Lua50ref "file:write")
fprintf = function (f, fmt, ...)
    f:write(format(fmt, unpack(arg)))
  end

printpairs = function (pairs)
    for i=1,getn(pairs) do
      print(pairs[i][1], pairs[i][2])
    end
  end
fprintpairs = function (f, pairs)
    for i=1,getn(pairs) do
      fprintf(f, "%d %d\n", pairs[i][1], pairs[i][2])
    end
  end

-- (find-angg "LUA/lua50init.lua" "readfile" "writefile")
fnameprintpairs = function (fname, pairs)
    local f = assert(io.open(fname, "w+"))
    fprintpairs(f, pairs)
    f:close()
  end

bigfoo = function (fname_t, fname_gi, out_prefix, out_suffix)
    local tfile = read_tfile("78032.t")
    table.sort(tfile, comp12)
    for n in io.lines(fname_gi) do
      local gi = "gi|"..n
      local outfname = out_prefix..n..out_suffix
      print("writing:", outfname)
      local intervals = tfile_to_qstartqends(tfile, gi)
      local ovpintervals = qstartqends_to_ovps(intervals) 
      fnameprintpairs(outfname, ovpintervals)
    end
  end

PP(arg)
bigfoo(arg[1], arg[2], arg[3], arg[4])

%%%

# lua50 t.lua | less
lua50 t.lua 78032.f 78032.gi /tmp/ .foo

#




--[[
tfile = read_tfile("78032.t")
table.sort(tfile, comp12)
qstartqends = tfile_to_qstartqends(tfile, "gi|78032")
printpairs(qstartqends)
print("---")
ovps = qstartqends_to_ovps(qstartqends)
printpairs(ovps)
--]]
