## A package for categorical diagrams that understands ASCII art

(Lua)(La)TeX treats lines starting with "`%`" as comments, and ignores
them. This means that we can put anything we want in these "`%`"
lines - even code to be processed by other programs besides *TeX.
[Dednat6](http://angg.twu.net/dednat6.html) is a Lua program that
treats blocks of comments in a `.tex` file starting with "`%D`" as
specifications for categorical diagrams, blocks of comments starting
with "`%:`" as specifications for derivation trees, and blocks of
comments starting with "`%L`" as Lua code. Blocks of "`%L`" lines can
be used to extend Dednat6, and to inspect and to change the current
data structures.

Here is an example. Suppose that you have a file called
`naturaltransformation.tex` containing:

```
\documentclass{article}
  %\usepackage{proof}
  \input diagxy
  %\xyoption{curve}
\begin{document}
  \catcode`\^^J=10
  \directlua{dofile "dednat6load.lua"}

%D diagram T:F->G
%D 2Dx     100 +20 +20
%D 2D  100     A
%D 2D         /|\
%D 2D        v v v
%D 2D  +30 FA --> GA
%D 2D
%D (( A FA |-> A GA |->
%D    FA GA -> .plabel= b TA
%D    A FA GA midpoint -->
%D ))
%D enddiagram
%D
$$\pu
  \diag{T:F->G}
$$

\end{document}
```

To compile it you need to have a copy of
[dednat6load.lua](http://angg.twu.net/dednat6/dednat6load.lua.html)
and of [dednat6/](http://angg.twu.net/dednat6/dednat6/) in the current
directory. If you run:

```
lualatex naturaltransformation.tex
```

you will get:

![](http://angg.twu.net/IMAGES/dednat6-TFG.png)


The "`\pu`" in the TeX code means "process until", or, more precisely,
"make Dednat6 process everything in the current `.tex` file until the
current point that it hasn't processed yet, and output the resulting
definitions"... this means that everything in the current `.tex` file
until its last `\pu` is processed twice, first by LaTeX, that ignores
the comments, and then by Dednat6, that only pays attention to the
lines with comments starting with "`%D`", "`%:`", or "`%L`"...

The block of "`%D`" lines in the example above is interpreted by
stack-based language slightly reminiscent of Forth, and it produces a
"definition" for the diagram `\diag{T:F->G}` - the one below,
containing (human-unfriendly) code for Michael Barr's
[diagxy](http://tug.ctan.org/tex-archive/macros/latex/contrib/xypic/doc/barrdoc.pdf)
package:

```
\defdiag{T:F->G}{
 \morphism(300,0)/->/%
  <-300,-375>[{A}`{FA};{}]
 \morphism(300,0)/->/%
  <300,-375>[{A}`{GA};{}]
 \morphism(0,-375)|b|/->/%
  <600,0>[{FA}`{GA};{TA}]
 \morphism(300,0)/|->/%
  <0,-375>[{A}`{\phantom{O}};{}]
}
```

Note that this part of the specification of our diagram

```
%D 2Dx     100 +20 +20
%D 2D  100     A
%D 2D         /|\
%D 2D        v v v
%D 2D  +30 FA --> GA
```

is a grid with coordinates and nodes: it defines a node "`A`" with
coordinates (120,100), a node "`FA`" at (100,130), and a node "`GA`"
at (140,130). The second part

```
%D (( A FA |-> A GA |->
%D    FA GA -> .plabel= b TA
%D    A FA GA midpoint -->
%D ))
```

draws arrows between these nodes using a stack-based language.

For more information see:

* the [home page of Dednat6](http://angg.twu.net/dednat6.html),
* these [slides of a presentation](http://angg.twu.net/dednat6/tug-slides.pdf),
* the [TUGboat article about it](https://tug.org/TUGboat/tb39-3/tb123ochs-dednat.pdf),
* the [source code](http://angg.twu.net/dednat6.html#a-big-example) for the big diagram below:

![](http://angg.twu.net/IMAGES/BCCL-std.png)

Dednat6 is very easy to hack and to extend (see
[this](http://angg.twu.net/dednat6.html#hacking)!) and it _should_ be
easy to implement other backends for 2D diagrams - for example, a
backend that uses Tikz... but I would need help with the Tikz syntax.
If you have practice with drawing categorical diagrams with Tikz and
would like to help, please get in touch! I'm eduardoochs at gmail dot
com.
