(*
 * This file:
 *   http://angg.twu.net/OCAML/firsthour.ml.html
 *   http://angg.twu.net/OCAML/firsthour.ml
 *           (find-angg "OCAML/firsthour.ml")
 * Author: Eduardo Ochs <eduardoochs@gmail.com>
 *
 * (find-es "ocaml" "first-hour")
 * (defun e () (interactive) (find-angg "OCAML/firsthour.ml"))
 *
 * (find-firsthourpage)
 * (find-firsthourtext)
 *
*)




(*
# 50 * 50;;
- : int = 2500

This is how it looks using utop:


 Welcome to utop version 2.7.0 (using OCaml version 4.13.1)! 


Type #utop_help for help about using utop.

( 10:12:16 )< command 0 >
utop # 50 * 50;;
- : int = 2500

The in-browser TryOCaml has a similar interface.

The examples in this tutorial can be typed in by hand, or copied into ocaml, utop
or TryOCaml with copy and paste. Alternatively, we may type into a file, and load
its contents directly with the #use directive:

$ ocaml
          OCaml version 4.13.1

# #use "program.ml"
Note that #use is not part of the OCaml language proper; it is an instruction to
the OCaml top level only.

Expressions
Our phrase 50 * 50 was an expression, which evaluated to 2500. OCaml told us
that the type was int, an integer (every expression in OCaml has a type). To avoid
repetition, we can give a name to our number:

# let x = 50;;
val x : int = 50
# x * x;;
- : int = 2500

Note that this is not a variable as in languages like C and Python. Its value cannot
be changed. We can write it all in one go using let ... in ...:

# let x = 50 in x * x;;
- : int = 2500

Of course, we can have multiple names:

# let a   = 1 in
  let b   = 2 in
    a +   b;;
- : int   = 3

Note that this is still just one expression. We can define a function to do the job
for any number:

# let square x = x * x;;
val square : int -> int = <fun>
# square 50;;
- : int = 2500

This says that square is a function with one argument, namely x, and that the
result of the function is the result of evaluating the expression x * x with the
given value associated with x. Here is another function, this time using the
comparison operator = to test for even-ness:

# let square_is_even x =
    square x mod 2 = 0;;
val square_is_even : int -> bool = <fun>
# square_is_even 50;;
- : bool = true
# square_is_even 3;;
- : bool = false
Notice the type OCaml infers for the function.

A function may take multiple arguments. Unlike in imperative languages, they are
written without parentheses and commas. We shall explain why later.

# let ordered a b c =
    a <= b && b <= c;;
val ordered : 'a -> 'a -> 'a -> bool = <fun>
# ordered 1 1 2;;
- : bool = true

We can work with floating-point numbers too, but we must write +., for example,
instead of just +:

# let average a b =
    (a +. b) /. 2.0;;
val average : float -> float -> float = <fun>

This is rather unusual. In other languages (such as C) integers get promoted to
floating point values in certain circumstances. For example if you write 1 + 2.5
then the first argument (which is an integer) is promoted to a floating point
number, and the result is also a floating point number.

OCaml never does implicit casts like this. In OCaml, 1 + 2.5 is a type error. The +
operator in OCaml requires two integers as arguments, and here we're giving it
an integer and a floating point number, so it reports this error:

# 1 + 2.5;;
Error: This expression has type float but an expression was expected of type
         int

OCaml doesn't promote integers to floating point numbers automatically so this is
also an error:

# 1 +. 2.5;;
Error: This expression has type int but an expression was expected of type
         float
  Hint: Did you mean `1.'?

What if you actually want to add an integer and a floating point number together?
(Say they are stored as i and f). In OCaml you need to explicitly cast:

float_of_int i +. f

The float_of_int function takes an int and returns a float.

You might think that these explicit casts are ugly, time-consuming even, but there
are several arguments in their favour. Firstly, OCaml needs this explicit casting to
be able to work out types automatically, which is a wonderful time-saving feature.
Secondly, if you've spent time debugging C programs you'll know that (a) implicit
casts cause errors which are hard to find, and (b) much of the time you're sitting
there trying to work out where the implicit casts happen. Making the casts
explicit helps you in debugging. Thirdly, some casts are actually expensive
operations. We do ourselves no favours by hiding them.

Recursive functions
A recursive function is one which uses itself in its own definition. An OCaml
function isn't recursive unless you explicitly say so by using let rec instead of
just let. Here's an example of a recursive function:

# let rec range a b =
    if a > b then []
    else a :: range (a + 1) b;;
val range : int -> int -> int list = <fun>
# let digits = range 0 9;;
val digits : int list = [0; 1; 2; 3; 4; 5; 6; 7; 8; 9]

We have used OCaml's if ... then ... else ... construct to test a condition and
choose a path of evaluation. Notice that, like everything else in OCaml, it is an
expression not a statement. The result of evaluating the whole expression is
either the result of evaluating the then part or the else part.

The only difference between let and let rec is in the scoping of the function
name. If the above function had been defined with just let, then the call to range
would have tried to look for an existing (previously defined) function called range,
not the currently-being-defined function.

Types
The basic types in OCaml are:

OCaml type    Range

int           63-bit signed int on 64-bit processors, or 31-bit signed int on
              32-bit processors
float         IEEE double-precision floating point, equivalent to C's double
bool          A boolean, written either 'true' or 'false'
char          An 8-bit character
string        A string (sequence of 8 bit chars)

OCaml provides a char type which is used for simple 8-bit characters, written 'x'
for example. There are comprehensive Unicode libraries which provide more
extensive functionality for text management.

Strings are not just lists of characters. They have their own, more efficient
internal representation. Strings are immutable.

When we type our expressions into the OCaml top level, OCaml prints the type:

#   1 + 2;;
-   : int = 3
#   1.0 +. 2.0;;
-   : float = 3.
#   false;;
-   : bool = false
#   'c';;
-   : char = 'c'
#   "Help me!";;
-   : string = "Help me!"

Each expression has one and only one type.

OCaml works out types automatically so you will rarely need to explicitly write
down the type of your functions. However, OCaml often prints out what it infers
are the types of your functions, so you need to know the syntax. For a function f
which takes arguments arg1, arg2, ... argn, and returns type rettype, the
compiler will print:

f : arg1 -> arg2 -> ... -> argn -> rettype

The arrow syntax looks strange now but later you'll see why it was chosen. Our
function average which takes two floats and returns a float has type:

average : float -> float -> float

The OCaml standard int_of_char casting function:

int_of_char : char -> int

We can now look at some of the properties which distinguish OCaml from other
languages:

     · OCaml is a strongly statically typed language. This means each expression
       has a type, and only one type, and it is determined before the program is
       run.

     · OCaml uses type inference to work out (infer) the types, so you don't have
       to. If you use the OCaml top level, then OCaml will tell you its inferred type
       for your function.

     · OCaml doesn't do any implicit conversion of types. If you want a floating
       point number, you have to write 2.0 because 2 is an integer. OCaml does no
       automatic conversion between int and floats or any other type. As a side-
       effect of type inference in OCaml, functions (including operators) can't
      have overloaded definitions.

Pattern matching
Instead of using one or more if ... then ... else ... constructs to make choices in
OCaml programs, we can use the match keyword to match on multiple possible
values. Consider the factorial function:

# let rec factorial n =
    if n <= 1 then 1 else n * factorial (n - 1);;
val factorial : int -> int = <fun>

We may write it using pattern matching instead:

# let rec factorial n =
    match n with
    | 0 | 1 -> 1
    | x -> x * factorial (x - 1);;
val factorial : int -> int = <fun>

Equally, we could use the pattern _ which matches anything, and write:

# let rec factorial n =
    match n with
    | 0 | 1 -> 1
    | _ -> n * factorial (n - 1);;
val factorial : int -> int = <fun>

In fact, we may simplify further with the function keyword which introduces
pattern-matching directly:

# let rec factorial = function
    | 0 | 1 -> 1
    | n -> n * factorial (n - 1);;
val factorial : int -> int = <fun>

We will use pattern matching more extensively as we introduce more complicated
data structures.

Lists
Lists are a common compound data type in OCaml. They are ordered collections
of elements of like type. Here are some lists:

# [];;
- : 'a list = []
# [1; 2; 3];;
-   : int list = [1; 2; 3]
#   [false; false; true];;
-   : bool list = [false; false; true]
#   [[1; 2]; [3; 4]; [5; 6]];;
-   : int list list = [[1; 2]; [3; 4]; [5; 6]]

Each list can have a head (its first element) and a tail (the list of the rest of its
elements). There are two built-in operators on lists. The :: or cons operator, adds
one element to the front of a list. The @ or append operator combines two lists:

#   1 :: [2; 3];;
-   : int list = [1; 2; 3]
#   [1] @ [2; 3];;
-   : int list = [1; 2; 3]

We can write functions which operate over lists by pattern matching:

# let rec total l =
    match l with
    | [] -> 0
    | h :: t -> h + total t;;
val total : int list -> int = <fun>
# total [1; 3; 5; 3; 1];;
- : int = 13

You can see how the pattern h :: t is used to deconstruct the list, naming its
head and tail. If we omit a case, OCaml will notice and warn us:

# let rec total_wrong l =
    match l with
    | h :: t -> h + total_wrong t;;
Warning 8: this pattern-matching is not exhaustive.
Here is an example of a case that is not matched:
[]
val total_wrong : int list -> int = <fun>
# total_wrong [1; 3; 5; 3; 1];;
Exception: Match_failure ("", 2, 2).

We shall talk about the "exception" which was caused by our ignoring the warning
later. Consider now a function to find the length of a list:

# let rec length l =
    match l with
    | [] -> 0
    | _ :: t -> 1 + length t;;
val length : 'a list -> int = <fun>

This function operates not just on lists of integers, but on any kind of list. This is
indicated by the type, which allows its input to be 'a list (pronounced alpha
list).

#   length [1; 2; 3];;
-   : int = 3
#   length ["cow"; "sheep"; "cat"];;
-   : int = 3
#   length [[]];;
-   : int = 1

Why is this? Because in the pattern _ :: t the head of the list is not inspected, so
its type cannot be relevant. Such a function is called polymorphic. Here is another
polymorphic function, our own version of the @ operator for appending:

# let rec append a b =
    match a with
    | [] -> b
    | h :: t -> h :: append t b;;
val append : 'a list -> 'a list -> 'a list = <fun>

Can you see how it works? Notice that the memory for the second list is shared,
but the first list is effectively copied. Such sharing is common when we use
immutable data types (ones whose values cannot be changed).

We might wish to apply a function to each element in a list, yielding a new one.
We shall write a function map which is given another function as its argument -
such a function is called "higher-order":

# let rec map f l =
    match l with
    | [] -> []
    | h :: t -> f h :: map f t;;
val map : ('a -> 'b) -> 'a list -> 'b list = <fun>

Notice the type of the function f in parentheses as part of the whole type. This
map function, given a function of type 'a -> 'b and a list of 'as, will build a list of
'bs. Sometimes 'a and 'b might be the same type, of course. Here are some
examples of using map:

#   map total [[1; 2]; [3; 4]; [5; 6]];;
-   : int list = [3; 7; 11]
#   map (fun x -> x * 2) [1; 2; 3];;
-   : int list = [2; 4; 6]

(The syntax fun ... -> ... is used to build a function without a name - one we will
only use in one place in the program.)

We need not give a function all its arguments at once. This is called partial
application. For example:
# let add a b = a + b;;
val add : int -> int -> int = <fun>
# add;;
- : int -> int -> int = <fun>
# let f = add 6;;
val f : int -> int = <fun>
# f 7;;
- : int = 13

Look at the types of add and f to see what is going on. We can use partial
application to add to each item of a list:

# map (add 6) [1; 2; 3];;
- : int list = [7; 8; 9]

Indeed we can use partial application of our map function to map over lists of lists:

# map (map (fun x -> x * 2)) [[1; 2]; [3; 4]; [5; 6]];;
- : int list list = [[2; 4]; [6; 8]; [10; 12]]

Other built-in types
We have seen basic data types like int, and our first compound data type, the list.
There are two more ways compound data types of interest. First we have tuples,
which are fixed length collections of elements of any type:

# let t = (1, "one", '1');;
val t : int * string * char = (1, "one", '1')

Notice how the type is written. Records are like tuples, but they have named
elements:

# type person =
    {first_name : string;
     surname : string;
     age : int};;
type person = { first_name : string; surname : string; age : int; }
# let frank =
    {first_name = "Frank";
     surname = "Smith";
     age = 40};;
val frank : person = {first_name = "Frank"; surname = "Smith"; age = 40}
# let s = frank.surname;;
val s : string = "Smith"

*)



