;; This is roughly the same as
;; (find-eev "eev-links.el")
;; but with a better syntax, with the low-level functions separated
;; from the high-level ones (the HL ones haven't been written yet),
;; and lots of room for nice extensions in ee-goto-args and
;; ee-goto-0args.

;; Edrx, 2003sep09
;; (font-lock-mode)

(defun ee-goto-fname (fname) (find-file (substitute-in-file-name fname)))
(defun ee-goto-node (node) (Info-goto-node node))
(defun ee-goto-sh (sh)
  (if (get-buffer sh)			; if the buffer already exists
      (switch-to-buffer sh)		; then just switch to it
    (switch-to-buffer sh)		; otherwise create it
    (insert (shell-command-to-string sh)) ; prepare its contents
    (goto-char (point-min))))	      ; and jump back to its beginning
;; to do: ee-goto-esh
;; (find-eev "eev-hybrids.el" "find-man.el")

;;;; ee-goto-args and ee-goto-0args:
;;;; moving to a specific position in a buffer

(defun ee-goto-0 ()
  (goto-char (point-min)))
(defun ee-goto-forwardstring (forwardstring)
  (search-forward forwardstring))
(defun ee-goto-0string (0string)
  (ee-goto-0)
  (ee-goto-forwardstring forwardstring))

(defun ee-goto-line (line)
  (ee-goto-0)
  (forward-line (1- pos-spec)))
(defun ee-goto-lines (lines)
  (forward-line lines))

(defun ee-goto-args (args)
  (if args
      (let ((arg (car args)) (rest (cdr args)))
	(cond ((numberp arg) (ee-goto-lines arg) (ee-goto-args rest))
	      ((stringp arg) (ee-goto-forwardstring arg) (ee-goto-args rest))
	      (t (error "Strange ee-args: %S" args))))))
(defun ee-goto-0args (0args)
  (if 0args
      (let ((0arg (car 0args)) (rest (cdr 0args)))
	(cond ((numberp 0arg) (ee-goto-line 0arg) (ee-goto-args rest))
	      ((stringp 0arg) (ee-goto-0string 0arg) (ee-goto-args rest))
	      (t (error "Strange ee-0args: %S" 0args))))))

;;;; Links to Emacs Lisp things
;;; Tests:
;; (ee-goto-efunctiondescr 'describe-function)
;; (ee-goto-evariabledescr 'ctl-x-map)
;; (ee-goto-ekeydescr "\C-h\C-k")
;; (ee-goto-efunction 'describe-function)
;; (ee-goto-evariable 'ctl-x-map)

(defun ee-goto-efunctiondescr (efunction)
  (describe-function efunction)
  efunction)
(defun ee-goto-evariabledescr (evariable)
  (describe-variable evariable)
  evariable)
(defun ee-goto-ekeydescr (key)
  (describe-key key))

(defun ee-goto-efunction (efunction)
  (let ((location (find-function-noselect efunction)))
    (switch-to-buffer (car location))
    (goto-char (cdr location))))
(defun ee-goto-evariable (evariable)
  (let ((location (find-variable-noselect evariable)))
    (switch-to-buffer (car location))
    (goto-char (cdr location))))
;; to do: (defun ee-goto-ekey (key) ???)

;;;; Anchors, including some debian/dpkg specific ones

(defun ee-goto-anchor (anchor)
  (ee-goto-0string (format ee-anchor-format anchor)))
(defun ee-goto-Package (Package)
  (ee-goto-0string (format "\nPackage: %s\n" anchor)))

(defun ee-goto-anchorargs (anchorargs)
  (when anchorargs
    (if (car anchorargs)
	(ee-goto-anchor (car anchorargs))
      (ee-goto-0))
    (ee-goto-args (cdr anchorargs))))
(defun ee-goto-Packageargs (Packageargs)
  (when Packageargs
    (if (car Packageargs)
	(ee-goto-Package (car Packageargs))
      (ee-goto-0))
    (ee-goto-args (cdr Packageargs))))


