# (find-angg "TODO" "peter_russell_demo")
# (find-angg ".emacs" "dynkin-diags")


# Constants
#
set dyncorner(...,-1,0) "-10 0"
set dyncorner(...,1,0)  "10 0"
set dyncorner(...,0,-1) "0 -10"
set dyncorner(...,0,1)  "0 10"
foreach {dir xness yness} {
  -1,-1 -2.5 -2.5    0,-1 0 -3   1,-1 2.5 -2.5
  -1,0  -3    0                  1,0  3    0  
  -1,1  -2.5  2.5    0,1  0  3   1,1  2.5  2.5 } {
  set dyncorner(O,$dir) "$xness $yness"
  set dyncorner(X,$dir) "$xness $yness"
  set dyncorner(.,$dir) "0 0"
}


# Variables
#
proc settmpvariables {xo yo} {
  global pmcommands xmin xmax ymin ymax
  global dynscale dynxorigin dynyorigin
  global XY dyntype
  set pmcommands ""
  set xmin ""; set xmax ""; set ymin ""; set ymax ""
  set dynscale 2.5
  set dynxorigin $xo; set dynyorigin $yo
  set XY(dummy) !;      unset XY;	# clear array
  set dyntype(dummy) !; unset dyntype;	# clear array
}
settmpvariables 0 0

proc origin {x y} {			# obsolete!
  global dynxorigin dynyorigin
  set dynxorigin $x
  set dynyorigin $y
}


# dynnode, dynline, dyntext
#
proc puts_pm {str} { global pmcommands; append pmcommands "$str\n" }
proc dynpmcoords {tag {dir {}}} {
  global XY dynxorigin dynyorigin dynscale dyntype dyncorner
  foreach {x y} $XY($tag) {}
  set x [expr ($x-$dynxorigin)*$dynscale]
  set y [expr ($y-$dynyorigin)*-$dynscale]
  if {$dir==""} {
    return "$x $y"
  } else {
    foreach {dx dy} $dyncorner($dyntype($tag),$dir) {}
    return "[expr $x+$dx] [expr $y+$dy]"
  }
}
proc dynput_pm {tag stuff} {
  foreach {x y} [dynpmcoords $tag] {}
  puts_pm "\\put($x,$y){$stuff}"
}
proc dyntext_pm {tag text} { dynput_pm $tag "\\makebox(0,0){$text}" }
proc dyndots_pm {tag} { dyntext_pm $tag "\$\\cdots\$" }
proc dyno_pm {tag} { dynput_pm $tag "\\dyno" }
proc dynx_pm {tag} { dynput_pm $tag "\\dynx" }

proc sign {n} { expr {$n<0?-1:($n>0?1:0)} }
proc signtowards {tag1 tag2} {
  global XY
  foreach {x1 y1 x2 y2} "$XY($tag1) $XY($tag2)" {}
  return "[sign [expr $x2-$x1]],[sign [expr $y1-$y2]]"
}
proc dynline_pm {x1 y1 x2 y2} {
  if {$x1>$x2} {
    dynline_pm $x2 $y2 $x1 $y1
  } elseif {$x1==$x2} {
    if {$y1>$y2} {
      puts_pm "\\put($x1,$y1){\\line(0,-1){[expr $y1-$y2]}}"
    } else {  # $y1<$y2
      puts_pm "\\put($x1,$y1){\\line(0,1){[expr $y2-$y1]}}"
    }
  } else {    # $x1<$x2
    puts_pm "\\put($x1,$y1){\\line(1,[sign [expr $y2-$y1]]){[expr $x2-$x1]}}"
  }
}
proc dynline {tag1 tag2 args} {
  foreach {x1 y1} [dynpmcoords $tag1 [signtowards $tag1 $tag2]] {}
  foreach {x2 y2} [dynpmcoords $tag2 [signtowards $tag2 $tag1]] {}
  dynline_pm $x1 $y1 $x2 $y2
  if {$args!=""} { eval dynline $tag2 $args }
}

proc min {a b} { expr {$a<$b?$a:$b} }
proc max {a b} { expr {$a>$b?$a:$b} }
foreach {xmin xmax ymin ymax} {0 0 0 0} {}
proc setXY {tag x y} {
  global XY xmin xmax ymin ymax
  set XY($tag) "$x $y"
  if {$xmin==""} { # first time: no xmin/ymin/xmax/ymax
    set xmin $x; set xmax $x
    set ymin $y; set ymax $y
  } else {
    set xmin [min $xmin $x]
    set ymin [min $ymin $y]
    set xmax [max $xmax $x]
    set ymax [max $ymax $y]
  }
}
proc dynnode {tag type x y args} {
  global XY dyntype
  setXY $tag $x $y
  set dyntype($tag) $type
  switch -exact $type {
    O   { dyno_pm $tag }  
    X   { dynx_pm $tag }  
    ... { dyndots_pm $tag }
    .   { }
    default { error }
  }
  if {$args!=""} { eval dynnode $args }
}
proc dyntext {tag text x y args} {
  global XY
  setXY $tag $x $y
  dyntext_pm $tag $text
  if {$args!=""} { eval dyntext $args }
}



proc putsorigin {} {			# obsolete
  global xmax xmin ymax ymin
  puts "% origin $xmin $ymax"
}
proc beginpicture {} {
  global xmax xmin ymax ymin dynscale
  set xsize [expr ($xmax-$xmin+10)*$dynscale]
  set ysize [expr ($ymax-$ymin+10)*$dynscale]
  return "\\begin{picture}($xsize,$ysize)"
}
proc putsbeginpicture {} { puts [beginpicture] }



# rundynkinblock: run the code for a diagram two times: the first time
# just to get the bounding box, and the second time "seriously", with
# the origin set to the lower left corner, to produce the "right"
# picture mode code.
#
proc rundynkinblock {string} {
  global xmin ymax pmcommands 
  settmpvariables 0 0
  uplevel #0 $string
  settmpvariables $xmin $ymax
  uplevel #0 $string
  return "[beginpicture]\n$pmcommands\\end{picture}\n"
}

# rundynkingblocks: take a "big string" - the contents of a file with
# embedded code chunks describing Dynkin diagrams -, and run each
# chunk separately to produce another big string, this one with a
# series of \dynkindiagdef's.
#
proc before_regexp {re string} {
  regexp -indices $re $string where
  string range $string 0 [expr [lindex $where 0]-1]
}
proc rundynkinblocks {bigstring} {
  global xmin xmax ymin ymax
  set rest $bigstring
  set regexp1 "\n(%DD:\[ \t\]*(\[^ \t\n\]+))\[ \t\]*(\n.*)"
  set regexp2 "\n%DD\[ \t\]*\n(.*)"
  set alldiagdefs ""
  while {[regexp $regexp1 $rest -> diagnameline diagname newrest]} {
    puts -nonewline "Processing $diagname..."
    if {![regexp $regexp2 $newrest -> newnewrest]} {
      puts "\nThe diagram block that starts with \"$diagnameline\" is bad -"
      puts "it doesn't have a closing \"%DD\"."
      exit 1
    } else {
      set percentedcode [before_regexp $regexp2 $newrest]
      if {[regexp "\n\[^%\]" $percentedcode]} {
	puts "\nThe diagram block that starts with \"$diagnameline\" is bad -"
	puts "it should have a series of (Tcl) lines all starting with \"%\","
	puts "then a \"%DD\" to close."
	exit 1
      }
      regsub -all "\n%" $percentedcode "\n" code
      append alldiagdefs "%\n"
      append alldiagdefs "\\dynkindiagdef{$diagname}{\n"
      append alldiagdefs [rundynkinblock $code]
      append alldiagdefs "}\n"
      puts " ok: xmin ymin xmax ymax = $xmin $ymin $xmax $ymax."
    }
    set rest $newnewrest
  }
  return $alldiagdefs
}



# convert: read a LaTeX with embedded dynkin-diag code chunks, and
# produce the corresponding \input'able file.
#
proc readfile {fname} {
  set f [open $fname r]; set s [read $f]; close $f
  return $s
}
proc writefile {fname s} {
  set f [open $fname w]; puts -nonewline $f $s; close $f
}
proc convert {fnamein fnameout} {
  set h "tclsh [info script] $fnamein $fnameout"
  set header "% Generated automatically by:\n%   $h\n% Do not edit by hand!\n"
  writefile $fnameout "$header[rundynkinblocks [readfile $fnamein]]"
}

# Top level (what to do with the command-line arguments)
#
puts [join $argv]
eval [join $argv]




set ignorethis {
#
cd ~/dynkin/; tclsh dynkin.tcl 'rundynkinblocks [readfile dyd-ex.tex]'
#
cd ~/dynkin/
rm -v dyd-ex.pmd.tex
tclsh dynkin.tcl convert dyd-ex.tex dyd-ex.pmd.tex
read
l dyd-ex.pmdiags.tex
#
}
