#!/bin/awk -f
# URLs for this file:
#   http://angg.twu.net/eev/changer.awk
#   http://angg.twu.net/eev/changer.awk.html
# Version: 2002jul15; copyleft Edrx. License: GPL.
# Please help me with the headers! :)
#
# changer.awk: an awk script that is used by changer.sh and
# "make dotfiles-install" to modify initialization files.
# (find-eev "changer.sh")
# (find-eev "Makefile" "dotfiles")
# (find-eev "INSTALL" "eevblock-file-format")
#
# Usage (example):
#
#   awk -f changer.awk \
#       -v eevblock=rcfiles/.bashrc \
#       ~/tmp/.bashrc > ~/tmp/.bashrc.new
#
# The current "eevblock" files:
# (find-eev "rcfiles/")
# (find-eev "rcfiles/.bashrc")
# (find-eev "rcfiles/.cshrc")
# (find-eev "rcfiles/.zshrc")
# (find-eev "rcfiles/.emacs")
# (find-eev "rcfiles/.gdbinit")

# If you have never seen an Awk program before, the following links
# should take you to the pages in the Gawk Manual that describe all
# the features used in this script.
#
# (find-node "(gawk)Executable Scripts" "#! /bin/awk -f")
# (find-node "(gawk)Running gawk" "awk -f PROGRAM-FILE INPUT-FILE1")
# (find-node "(gawk)Assignment Options" "-v VARIABLE=TEXT")
# (find-node "(gawk)Very Simple")
# (find-node "(gawk)Pattern Overview" "`BEGIN'")
# (find-node "(gawk)Getline/Variable/File" "`getline VAR < FILE'")
# (find-node "(gawk)Typing and Comparison" "`X == Y'")
# (find-node "(gawk)While Statement")
# (find-node "(gawk)Basic Printf" "printf FORMAT, ITEM1, ITEM2, ...")
# (find-node "(gawk)Redirection" "`print ITEMS > OUTPUT-FILE'")
# (find-node "(gawk)Special FD" "`/dev/stderr'")
# (find-node "(gawk)Exit Statement")
# (find-node "(gawk)Using Variables" "initialized to")
# (find-node "(gawk)Statements/Lines" "continue")
# (find-node "(gawk)Fields" "`$0'")
#
# (find-node "(gawk)Index")

BEGIN {
  # eevblock_file
  # Process the eevblock file:
  # the first two lines of it give the beg_eev_block and end_eev_block
  # strings, then come any number of comment lines, then a line saying
  # *exactly* "::end of comments::"; the rest of the eevblock file
  # (and we will leave the BEGIN block before starting to read it) is
  # what will go between the "beg_eev_block" and "end_eev_block" lines
  # in the rc file that will be modified.

  getline beg_eev_block < eevblock;
  getline end_eev_block < eevblock;
  while (1) {
    gotline = (getline line < eevblock);
    if (!gotline) {
      printf "File %s ended before the \"::end of comments::\" line\n",
	eevblock > "/dev/stderr";
      exit 1;
    }
    if (line == "::end of comments::") {
      break;
    }
  }
}

state == 0 {			# if we're before the first beg_eev_block line
  if ($0 != beg_eev_block) {
    print;			# then echo every line;
  } else {			# but if we meet the beg_eev_block line,
    print;			# echo it,
    state = 1;			# change to state 1,
    while (getline line < eevblock) {
      print line;		# immediately dump the rest of the
    }				# eevblock file, and
    next;			# start processing the next line (in state 1).
  }
}

state == 1 {			# in state 1 skip all the lines until
  if ($0 == end_eev_block) {	# we meet the "end_eev_block" line,
    print;			# that should be echoed.
    state = 2;			# On meeting it set the state to 2
    next;			# and start processing the next line w/state=2.
  }
}

state == 2 {			# in state 2 echo everything that comes.
  print				# Further beg_eev_block/end_eev_block lines
}				# aren't treated specially.

END {				# on end of input,
  if (state == 0) {		# if we have never met a beg_eev_block,
    print "";			# print a blank line, as a separator,
    print beg_eev_block;	# then print a beg_eev_block line,
    while (getline line < eevblock) {
      print line;		# then dump the contents of the eevblock file,
    }
    print end_eev_block;	# then print an end_eev_block line.
  }
  if (state == 1) {		# if we had a beg_eev_block but it never
    printf "The file being modified had an \"%s\"\n" \
      "but no closing \"%s\"!\n", beg_eev_block, end_eev_block \
      > "/dev/stderr";		# closed, signal an error.
    exit 2
  }
}
