;; Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.

;; Author:     Eduardo Ochs <edrx@mat.puc-rio.br>
;; Maintainer: Eduardo Ochs <edrx@mat.puc-rio.br>
;; Version:    2002may11
;; Keywords:   escripts, hyperlinks, shell, tcl, perl, tex
;;
;; This file was copylefted to prevent against patent psychopaths; if
;; you want a version with any other license you'll have to write it
;; yourself. More formally,
;;
;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 2 of the
;; License, or (at your option) any later version.
;;
;; URLs for this file:
;;   http://angg.twu.net/eev/eev-misc.el
;;   http://angg.twu.net/eev/eev-misc.el.html

;;; Commentary:

;; This package provides support for e-scripts in Emacs. Look in
;; <http://angg.twu.net/emacs.html> for more information and in
;; <http://angg.twu.net/escripts.html> for examples of e-scripts... an
;; e-script is typically a text file in which certain parts are meant
;; to be executed by the shell or by other programs and certain
;; comments are in fact Emacs Lisp code; executing one of these
;; comments with C-x C-e (i.e., eval-last-sexp) brings up a file or an
;; info node, and thus works as a hyperlink. Many of the comments in
;; this file can be executed in this way.

;; See also the "eev Manifesto", at
;; <http://angg.twu.net/eev-manifesto.html>, the section named
;; "eev-howto", below, and whatever else seem important in
;; <http://angg.twu.net/>; that site is almost completely dedicated to
;; using eev.el.

;;;;
;;;; code-c-d-examples  (to ".code-c-d-examples")
;;;; Some examples of code-c-d's.
;;;; (given with Debian 3.0 in mind).
;;;;
;;
;; Note: if you run `(code-c-d PREFIX ...)' twice with the same
;; PREFIX, the second run overrides the first.  So it is quite trivial
;; to recycle the prefixes used in these examples...
;;
;; Note also that a "typical" .emacs (like mine! :-) has over 150
;; calls to code-c-d:
;;
;; (find-angg ".emacs" "code-c-ds")

;; e
;; Define find-efile, find-enode, etc.
;; I'm using ugly hacks here, and `locate-library' is a slow
;; function... please, if you have any ideas on how to make this
;; better, get in touch!!! Another problem: in some OSs and
;; distributions the info file for Emacs has a name different from
;; just "emacs" - for example in Debian 2.2 it will be either
;; "emacs-e19" or "emacs-e20", because the two versions can coexist,
;; each one with its manual, and the info-path is the same. In such
;; cases you'll have to write the right call to code-c-d yourself,
;; changing the last parameter.
;;
(defvar ee-emacs-lisp-lisp-main-directory
  (or (file-name-directory (locate-library "loadup.el"))
      (format "/usr/share/emacs/%d.%d/lisp/"
	      emacs-major-version emacs-minor-version))
  "A directory like \"/usr/share/emacs/<emacs-version>/lisp/\".
See the source code.
The method used to calculate the value of this variable is crude,
slow, ugly, and doesn't always work.")

(code-c-d "e" ee-emacs-lisp-lisp-main-directory "emacs")
(code-c-d "el" ee-emacs-lisp-lisp-main-directory "elisp")

(code-c-d "eev" "~/eev/")
(code-c-d "es" "$ES/")			; my e-scripts repository
(code-c-d "shttp" "$S/http/")		; local copies of pages from the web
(code-c-d "sftp" "$S/ftp/")		; same

(code-c-d "knuth" "$SCTAN/systems/knuth/") ; TeXbook, METAFONT book
(code-c-d "k2" "/usr/src/linux-2.0/")
(code-c-d "k22" "/usr/src/linux-2.2/")
(code-c-d "eli" "/usr/src/emacs-lisp-intro-1.05/" "emacs-lisp-intro")
(code-c-d "eetc" "/usr/share/emacs/20.7/etc/" "emacs-e20")
(code-c-d "e19" "/usr/share/emacs/19.34/lisp/" "emacs-19/emacs")
;;(code-c-d "e" "/usr/share/emacs/19.34/lisp/" "emacs-19/emacs")
;;(code-c-d "el" "/usr/share/emacs/19.34/lisp/" "elisp")

(code-c-d "ud" "/usr/doc/")
(code-c-d "ht" "/usr/doc/HOWTO/")
(code-c-d "htet" "/usr/doc/HOWTO/en-txt/")
(code-c-d "drfc" "/usr/doc/doc-rfc/all-included-rfcs/")
(code-c-d "sagh" "/usr/share/doc/sysadmin-guide/sag.html/")
(code-c-d "nagh" "/usr/doc/ldp-nag/network-guide-1.0.html/")
(code-c-d "pack" "/usr/share/doc/packaging-manual/")
(code-c-d "packh" "/usr/share/doc/packaging-manual/packaging.html/")
(code-c-d "devref" "/usr/doc/developers-reference/")
(code-c-d "devrefh" "/usr/doc/developers-reference/developers-reference.html/")
(code-c-d "debfaq" "/usr/share/doc/debian/FAQ/")
(code-c-d "debfaqh" "/usr/share/doc/debian/FAQ/")
(code-c-d "maintg" "/usr/doc/maint-guide/")
(code-c-d "maintgh" "/usr/doc/maint-guide/")
(code-c-d "dpol" "/usr/doc/debian-policy/")
(code-c-d "vldi" "/var/lib/dpkg/info/")

;; These two are just to make some links shorter:
;;
(defun find-devreftxt (&rest rest)
  "A hyperlink function to points of the \"Debian Developers Reference\" manual."
  (apply 'find-devreffile "developers-reference.txt.gz" rest))

(defun find-pl5pod (suffix &optional line)
  "A hyperlink function to the .pod files that make the manpages for Perl5."
  (find-fline (format "/usr/lib/perl5/5.004/pod/perl%s.pod" suffix) line))



;; shortcut-find-xxxs
;; I use these three functions to make hyperlinks to some files from my
;; home machine, when I want the hyperlinks to continue working after
;; converting the files to HTML. See <http://angg.twu.net/>.
;; I have included them here because `find-eev' will work in any
;; machine where eev is installed, and find-es and find-angg will work
;; anywhere my tourism package is installed.
;;
(defun find-es (file &rest rest)
  "A hyperlink function that points to anchor strings in e-script files."
  (apply 'find-anchor (ee-esfile (concat file ".e")) (car rest) (cdr rest)))
(defun find-angg (file &rest rest)
  "A hyperlink function to anchors in files in my (edrx) home directory."
  (apply 'find-anchor (concat "~/" file) rest))
(defun find-eev (file &rest rest)
  "A hyperlink function to anchors in files in ~/eev/."
  (apply 'find-anchor (concat "~/eev/" file) rest))

;; ee-load-back  (to ".ee-load-back")
;; These are experimental functions for when we want to save a block
;; of text -- for example, TeX text containg pictures drawn using the
;; LaTeX picture environment --, then have some external program
;; modify the saved block, then import back the saved block into
;; Emacs. Actually these functions are SO experimental that I don't
;; remember why I've put them here, and I may remove them soon.

(defun ee-read-file-and-strip (fname prefix suffix)
  (let* ((contents
	  (with-temp-buffer
	    (insert-file-contents (substitute-in-file-name fname))
	    (buffer-substring 1 (point-max))))
	 (p1 (length prefix))
	 (p2 (- (length contents) (length suffix)))
	 (prefix-ok (compare-strings contents nil p1 prefix nil nil))
	 (suffix-ok (compare-strings contents p2 nil suffix nil nil)))
    (if (not (eq prefix-ok t))
	(error "Bad prefix: %S" (substring contents 0 p1)))
    (if (not (eq suffix-ok t))
	(error "Bad suffix: %S" (substring contents p2 (length contents))))
    (substring contents p1 p2)))

(defun ee-load-back (delim fname prefix suffix)
  (let* ((origpoint (point))
	 (beg (ee-search-backward delim))
	 (end (ee-search-forward  delim))
	 (newcontents (ee-read-file-and-strip fname prefix suffix))
	 (newend (+ beg (length newcontents)))
	 (newpoint (min newend origpoint)))
    (kill-region beg end)
    (goto-char beg)
    (insert newcontents)
    (goto-char newpoint)))

;; (find-eevfile "eev-all.el")
;; (find-eev "eev-all.el" "bounded_support")
;; (find-eev "eev-gud.el" "eeg_in_emacs")
;; (find-elnode "Text Comparison")
;; (find-elnode "Creating Strings")

;; (ee-load-back ee-delimiter-hash "$EE" "set -v\n" "\nset +v")






;;;;
;;;; eexxx_examples  (to ".eexxx_examples")
;;;; These are just meant as examples, and are commented out with "'"s.
;;;;
;;
;; gdb: there used to be an `eegdb' function, but `eeg' replaced it.
;; The trick is that if you put something like
;;   define ee
;;     source ~/bin/ee.generic
;;   end
;; in your .gdbinit then you can use "ee" inside gdb to invoke the
;; saved block (and regard `eeg', defined above, as a sort of
;; `eegdb'). Note that this will work inside Emacs (i.e., inside GUD).
;; (find-angg ".gdbinit")
;;
;; It is also possible to define functions to send lines of
;; ~/bin/ee.generic to GUD or other comint-based modes, one at a time;
;; there is an example at <http://angg.twu.net/eev-extras.el#eeg_in_emacs>.
;; (find-angg "eev-extras.el" "eeg_in_emacs")

;; perl, tcl/tk, expect:
;;
'(defun eep (s e) (interactive "r")
  (ee-write s e "#!/usr/bin/perl\n" "" "~/bin/eep" "0755"))
'(defun eew (s e) (interactive "r")
  (ee-write s e "#!/usr/bin/wish\n" "" "$EET" "0755"))
'(defun eet (s e) (interactive "r")
  (ee-write s e "#!/usr/bin/tclsh\n" "" "$EET" "0755"))
'(defun eex (s e) (interactive "r")
  (ee-write s e "#!/usr/bin/expect\n" "" "$EET" "0755"))
'(defun eextk (s e) (interactive "r")
  (ee-write s e "#!/usr/bin/expectk\n" "" "$EET" "0755"))

;; eev to some remote machines, using ange-ftp:
;; (find-enode "Remote Files")
;;
'(defun eevsaci (s e) (interactive "r")
  (eev s e "/edrx@$SACI:bin/ee.sh"))
'(defun eevp (s e) (interactive "r")
  (eev s e "/patna:/home/root/bin/ee.sh"))

;; eed, for dosemu. Aaargh!!
;;
'(defun ee-crlfify (s)
  "Change each LF in S into a CRLF."
  (if (string-match "\\([^\n]*\\)\n\\(\\)" s)
      (concat (match-string 1 s) "\r\n"
	      (crlfify (substring s (match-beginning 2))))
    s))
'(defun eed (s e)
  "Write the region into \"/D/ee.bat\", changing the line terminators to CRLF."
  (interactive "r")
  (write-region (ee-crlfify (ee-se-to-string s e)) nil "/D/ee.bat"))

;; For lynx.
'(defun eel (s e) (interactive "r")
  (ee-write s e "lynx " "\n"))








;; Local Variables:
;; coding:               no-conversion
;; ee-anchor-format:     "%s"
;; ee-charset-indicator: ""
;; End:

;;; eev-misc.el ends here
