;;; eev.el --- embed shell/tcl/perl code and elisp hyperlinks in plain text.
;; Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.

;; Author:     Eduardo Ochs <edrx@mat.puc-rio.br>
;; Maintainer: Eduardo Ochs <edrx@mat.puc-rio.br>
;; Version:    2001dec26
;; Keywords:   escripts, hyperlinks, shell, tcl, perl, tex
;;
;; This file was copylefted to prevent against patent psychopaths; if
;; you want a version with any other license you'll have to write it
;; yourself. More formally,
;;
;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 2 of the
;; License, or (at your option) any later version.

;;; Commentary:

;; This package provides support for e-scripts in Emacs. Look in
;; <http://angg.twu.net/emacs.html> for more information and in
;; <http://angg.twu.net/escripts.html> for examples of e-scripts... an
;; e-script is typically a text file in which certain parts are meant
;; to be executed by the shell or by other programs and certain
;; comments are in fact Emacs Lisp code; executing one of these
;; comments with C-x C-e (i.e., eval-last-sexp) brings up a file or an
;; info node, and thus works as a hyperlink. Many of the comments in
;; this file can be executed in this way.

;; See also the "eev Manifesto", at
;; <http://angg.twu.net/eev-manifesto.html>, the section named
;; "eev-howto", below, and whatever else seem important in
;; <http://angg.twu.net/>; that site is almost completely dedicated to
;; using eev.el.


;; Variables related to eev-save.el:
;; .ee-file			(to "ee-file")
;; .ee-file-generic		(to "ee-file-generic")
;; .ee-file-gdb		(to "ee-file-gdb")
;; .eelatex-vars		(to "eelatex-vars")
;; .eexxx-bounded-vars	(to "eexxx-bounded-vars")
;; .ee-delimiter-hash		(to "ee-delimiter-hash")
;; .ee-delimiter-percent	(to "ee-delimiter-percent")

;; .ee-arg			(to "ee-arg")
;; .ee-anchor-format		(to "ee-anchor-format")
;; .ee-comment-format		(to "ee-comment-format")




;;;;
;;;; variables  (to ".variables")
;;;; (describe-variable 'ee-file)
;;;; (describe-variable 'ee-file-generic)
;;;; (describe-variable 'ee-temp-bounded-function)
;;;;


;;;;
;;;; Variables related to eev-save.el
;;;;

;; ee-file  (to ".ee-file")
;;
(defvar ee-file "~/eev/ee.sh"
  "The file where `eev' saves its temporary shell script.
See the docs on the function `eev'.")

;; ee-file-generic  (to ".ee-file-generic")
;;
(defvar ee-file-generic "~/eev/ee.generic"
  "The file used for all \"ee\" commands based on the generic interface.
The command `eeg' saves the region in this file.\n
When you run an application through the \"eeg\" support script (written in
Expect) you are able to paste lines from this file to the application.")

;; ee-file-gdb  (to ".ee-file-gdb")
;;
(defvar ee-file-gdb "~/eev/ee.gdb"
  "The file used for saving temporary gdb (e-)scripts.")

;; eelatex-vars  (to ".eelatex-vars")
;; Variables used by `eelatex':
;;
(defvar eelatex-pre "\\documentclass{book}\n\\begin{document}\n")
(defvar eelatex-post "\n\\end{document}\n")
(defvar eelatex-file "~/tmp/tmp.tex")
(defvar eelatex-eevscript "cd ~/tmp/; latex tmp.tex")

;; eexxx-bounded-vars  (to ".eexxx-bounded-vars")
;; For `ee-bounded':
;;
(defvar ee-temp-bounded-function 'eev-bounded
  "The function that \\[ee-bounded] will execute.
All standard `eexxx-bounded' functions set this variable when they are
run; `eev-bounded' set it to `eev-bounded', `eeg-bounded' to
`eeg-bounded', and so on.")

(put 'ee-temp-bounded-function 'safe-local-variable t)

;; ee-delimiter-hash  (to ".ee-delimiter-hash")
;;
(defvar ee-delimiter-hash "\n#-\n"
  "The delimiter string used by `eev-bounded' and other similar functions.
All `eexxx-bounded' functions that are used for languages where
comments start with a `#' use this string.\n
The default value for this variable may change in future versions. For
more information on the best way to set a value for it, follow this
link: (find-enode \"File Variables\")")

;; ee-delimiter-percent  (to ".ee-delimiter-percent")
;;
(defvar ee-delimiter-percent "\n%-\n"
  "The delimiter string used by `eelatex-bounded' and similar functions.
All `eexxx-bounded' functions that are used for languages where
comments start with a `%' use this string.\n
The default value for this variable may change in future versions. For
more information on the best way to set a value for it, follow this
link: (find-enode \"File Variables\")")






;; ee-anchor-format  (to ".ee-anchor-format")
;;
(defvar ee-anchor-format "<<%s>>"
  "Hyperlink functions based on `find-anchor' use this string to make a string to search for.\n
The default value for this variable may change in future versions. For
more information on the best way to set a value for it, follow this
link: (find-enode \"File Variables\")")

;; ee-comment-format  (to ".ee-comment-format")
;;
(defvar ee-comment-format "# %s\n"
  "The format used by `inn' to insert \"find-xxxnode\" lines.")

;; ee-arg  (to ".ee-arg")
;;
(defvar ee-arg nil
  "A variable used to pass a parameter to hyperlink functions.
See the documentation for `end-of-line-then-eval-last-sexp' to
understand how the trick works.")




;; Local Variables:
;; coding:               no-conversion
;; ee-anchor-format:     "%s"
;; ee-charset-indicator: ""
;; End:

;;; eev.el ends here
